/**
 * 定义表格对象
 */
function table () {
    /**表格名称，当该表格获取完数据后，应该存储的对象名 */
    this.name = "";
    /**表格所在的XPath,由验证表格功能具体生成,如果为null，说明验证不成功 */
    this.xpath = null;
    /**所有的单元格信息*/
    this.cells = [];
    /**当前table是否通过了验证*/
    let _isVerified = false;
    /**
     * 新生成一个单元格
     * @param {any} xpath
     */
    this.newCell = function (xpath, name, type, types, title) {
        _isVerified = false;//加入了
        let newCell = new cell(xpath, name, type, types, title);
        this.cells.push(newCell);
    }
    /**
     * 根据传入的cell进行验证，如果验证成功，xpath自动变化
     * @returns {boolean} 返回true或false
     * */
    this.verify = function () {
        _isVerified = false;
        this.xpath = null;//防止反复验证，需要清空XPath
        if (this.cells.length < 1) return _isVerified;
        let tableXpathArray = [];
        {//获取所有单元格共同的父节点路径
            let allCellXpaths = this.cells.map(cell => cell.xpath.split('/'));// 分割每个路径为组成部分
            for (let i = 0; i < allCellXpaths[0].length; i++) {
                let currentPath = allCellXpaths[0][i];
                if (allCellXpaths.every(path => path[i] === currentPath)) { // 检查所有路径是否有这个部分
                    tableXpathArray.push(currentPath);
                } else { break; }//如果发现不同的，直接就可以退出了
            }
            //判断下tableXPathArray不能和某一个元素的路径完全相同，如果相同就应该去掉一级
            // 当 cells 数组只包含一个单元格时，需要特别处理
            if (this.cells.length === 1) {
                let singleCellPath = this.cells[0].xpath.split('/');
                if (tableXpathArray.length === singleCellPath.length) {
                    // 如果共同父节点路径与单个单元格的路径完全相同，去掉最后一个元素
                    tableXpathArray.pop();
                }
            }
        }
        if (tableXpathArray.length < 1) return _isVerified;//如果没有共同父节点，返回false
        while (tableXpathArray.length > 0) {
            this.cells.forEach(cell => cell.updateRelativePath(tableXpathArray));//先对所有的Cell更新下相对路径
            if (this.cells.every(cell => cell.isExist())) break;//如果每个cell都存在，成功
            if(tableXpathArray[tableXpathArray.length - 1]=='HTML') return
            tableXpathArray.pop();//向上一级找
        }
        if (tableXpathArray.length < 1) return _isVerified;//如果没有共同父节点，返回false
        this.cells.forEach(cell => cell.confirm());//对所有单元格进行确认
        _isVerified = true;
        this.xpath = tableXpathArray.join("/");//是否可以转换成SmallXPath
        return _isVerified;
    }
    /**
     * 加载JSON格式，生成一个新的对象结构
 * @param {table} obj 元素的XPath路径
     */

    this.load = function (obj) {
        this.name = obj.name;
        this.xpath = obj.xpath;
        for (var i = 0; i < obj.cells.length; i++) {
            let objCell = obj.cells[i];
            let newCell = new cell(objCell.xpath);
            newCell.type = objCell.type;
            newCell.tableXpath = objCell.tableXpath;
            newCell.relativeXpath = objCell.relativeXpath;
            this.cells.push(newCell);
        }
        _isVerified = true;
    }
    /**
     * 返回当前表格的数据
     */
    this.getData = function () {
        let rtns = [];//定义返回值
        if (!_isVerified) return null;//如果还没通过验证不返回数据
        let rowIndex = 1;
        while (true) {
            let result = {}; // 为每个 cell 创建一个新的结果对象
            this.cells.forEach(cell => {
                cell.getData(result, rowIndex); // 调用 getData 方法
            });
            if (Object.keys(result).length === 0) break;//不断的根据rowIndex取值，直到取不到值就退出，需要注意如果第0行是表头，也不一定可以取到值
            rtns.push(result);
            rowIndex++;
        }
        return rtns;
    }
}
/**
 * 定义的表格获取时要传入的对象信息
 * @param {string} xpath 元素的XPath路径
 * @param {string} name 元素取值后返回的属性名
 * @param {string} type 元素取值类型，文本，链接，元素路径（点击动作用）
 * @param {string} title 标题信息
 */
function cell (xpath, name, type, types, title) {
    /**当前单元格的绝对路径，存入Action时有用 */
    this.xpath = xpath;
    /**当前单元格存储数据时的属性名 */
    this.name = name;
    /**单元格取值类型， 文本，链接，元素路径（点击动作用）*/
    this.type = type;
    /**当前单元格所具备的可以操作的类型集合 */
    this.types = types;
    /** 单元格在表上的表头信息*/
    this.title = title;
    /**当表结构验证成功后，正确的表绝对路径,不包含下标 */
    this.tableXpath = null;
    /**当表结构验证成功后，正确的单元格相对路径 */
    this.relativeXpath = null;
    /**当前XPath的数组结构 */
    let _xpathArray = this.xpath.split('/');
    /**当前表路径的数组结构 */
    let _tableXpathArray = null;
    /**当前单元格的相对路径的数组结构 */
    let _relativeXpathArray = null;
    /**
     * 根据父路径更新当前的相对路径
     * @param {string[]} parentXpath 父路径
     */
    this.updateRelativePath = function (parentXpath) {
        _tableXpathArray = parentXpath;//其实这边严谨的来说要验证一下XPath是否真的有相同的根
        _relativeXpathArray = _xpathArray.slice(_tableXpathArray.length);
        // 移除最后一个元素的下标 
        if (_tableXpathArray.length > 0) {
            let lastElement = _tableXpathArray[_tableXpathArray.length - 1];
            let lastElementWithoutIndex = lastElement.split('[')[0];
            _tableXpathArray[_tableXpathArray.length - 1] = lastElementWithoutIndex;
        }
    }
    /**
     * 单元格是否存在
     * @returns {boolean}
     */
    this.isExist = function () {
        let existCount = 0;
        for (let i = 1; i < 3; i++) {
            // 验证是否能找到元素
            let cellXpath = _tableXpathArray.join('/') + '[' + i + ']' + '/' + _relativeXpathArray.join('/');
            let result = document.evaluate(cellXpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
            if (!!result.singleNodeValue) {
                existCount++;
            }
        }
        return existCount > 1;
    }
    /**当所有单元格验证成功后，由表发起确认动作 */
    this.confirm = function () {
        this.tableXpath = _tableXpathArray.join("/");//处理下标的事
        this.relativeXpath = _relativeXpathArray.join("/");
    }
    /**
     * 根据索引获取值,返回{name:value}格式
     * @param {object} result 需要赋值的对象
     * @param {int} index 索引号
     */
    this.getData = function (result, index) {
        let cellXpath = this.tableXpath + "[" + index + "]" + '/' + this.relativeXpath;
        let results = document.evaluate(cellXpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if (!!results.singleNodeValue) {
            if (this.type == 'xPath') {
                result[this.name] = xpath
            } else if (this.type == '文本') {
                result[this.name] = results.singleNodeValue['textContent'];
            } else {
                result[this.name] = results.singleNodeValue[this.type];
            }

        }
    };
    return this;//返回当前对象
}
export { cell, table }  